@@ -1076,7 +1076,7 @@ gotd_dispatch_listener(int fd, short event, void *arg)
 
 	if (event & EV_READ) {
 		if ((n = imsgbuf_read(ibuf)) == -1)
-			fatal("imsg_read error");
+			fatal("imsgbuf_read error");
 		if (n == 0) {
 			/* Connection closed. */
 			shut = 1;
@@ -1159,7 +1159,7 @@ gotd_dispatch_notifier(int fd, short event, void *arg)
 
 	if (event & EV_READ) {
 		if ((n = imsgbuf_read(ibuf)) == -1)
-			fatal("imsg_read error");
+			fatal("imsgbuf_read error");
 		if (n == 0) {
 			/* Connection closed. */
 			shut = 1;
@@ -1233,7 +1233,7 @@ gotd_dispatch_auth_child(int fd, short event, void *arg)
 
 	if (event & EV_READ) {
 		if ((n = imsgbuf_read(ibuf)) == -1)
-			fatal("imsg_read error");
+			fatal("imsgbuf_read error");
 		if (n == 0) {
 			/* Connection closed. */
 			shut = 1;
@@ -1404,7 +1404,7 @@ gotd_dispatch_client_session(int fd, short event, void *arg)
 
 	if (event & EV_READ) {
 		if ((n = imsgbuf_read(ibuf)) == -1)
-			fatal("imsg_read error");
+			fatal("imsgbuf_read error");
 		if (n == 0) {
 			/* Connection closed. */
 			shut = 1;
@@ -1565,7 +1565,7 @@ gotd_dispatch_repo_child(int fd, short event, void *arg)
 
 	if (event & EV_READ) {
 		if ((n = imsgbuf_read(ibuf)) == -1)
-			fatal("imsg_read error");
+			fatal("imsgbuf_read error");
 		if (n == 0) {
 			/* Connection closed. */
 			shut = 1;
