@@ -22,6 +22,7 @@
 #include <sys/time.h>
 #include <sys/mman.h>
 
+#include <err.h>
 #include <inttypes.h>
 #include <limits.h>
 #include <signal.h>
@@ -1066,15 +1067,18 @@ recv_object_id_queue(struct got_object_id_queue *queue,
 		for (i = 0; i < nids; i++) {
 			err = got_object_qid_alloc_partial(&qid);
 			if (err)
-				return err;
+				goto done;
 			memcpy(&qid->id, &ids[i], sizeof(qid->id));
 			STAILQ_INSERT_TAIL(queue, qid, entry);
 			err = got_object_idset_add(queued_ids, &qid->id, NULL);
 			if (err)
-				return err;
+				goto done;
 		}
+		free(ids);
+		ids = NULL;
 	}
-
+done:
+	free(ids);
 	return err;
 }
 
@@ -1565,6 +1569,8 @@ enumeration_request(struct imsg *imsg, struct imsgbuf *ibuf,
 			err = send_tree_enumeration_done(ibuf);
 			if (err)
 				goto done;
+			got_object_commit_close(commit);
+			commit = NULL;
 			continue;
 		}
 
@@ -1989,12 +1995,17 @@ main(int argc, char *argv[])
 
 	signal(SIGINT, catch_sigint);
 
-	imsg_init(&ibuf, GOT_IMSG_FD_CHILD);
+	if (imsgbuf_init(&ibuf, GOT_IMSG_FD_CHILD) == -1) {
+		warn("imsgbuf_init");
+		return 1;
+	}
+	imsgbuf_allow_fdpass(&ibuf);
 
 	err = got_object_cache_init(&objcache, GOT_OBJECT_CACHE_TYPE_OBJ);
 	if (err) {
 		err = got_error_from_errno("got_object_cache_init");
 		got_privsep_send_error(&ibuf, err);
+		imsgbuf_clear(&ibuf);
 		return 1;
 	}
 
@@ -2003,6 +2014,7 @@ main(int argc, char *argv[])
 	if (pledge("stdio recvfd", NULL) == -1) {
 		err = got_error_from_errno("pledge");
 		got_privsep_send_error(&ibuf, err);
+		imsgbuf_clear(&ibuf);
 		return 1;
 	}
 
@@ -2022,12 +2034,14 @@ main(int argc, char *argv[])
 	err = receive_packidx(&packidx, &ibuf);
 	if (err) {
 		got_privsep_send_error(&ibuf, err);
+		imsgbuf_clear(&ibuf);
 		return 1;
 	}
 
 	err = receive_pack(&pack, &ibuf);
 	if (err) {
 		got_privsep_send_error(&ibuf, err);
+		imsgbuf_clear(&ibuf);
 		return 1;
 	}
 
@@ -2148,10 +2162,11 @@ main(int argc, char *argv[])
 	commit_painting_free(&keep, &drop, &skip);
 	if (packidx)
 		got_packidx_close(packidx);
-	if (pack)
+	if (pack) {
 		got_pack_close(pack);
+		free(pack);
+	}
 	got_object_cache_close(&objcache);
-	imsg_clear(&ibuf);
 	if (basefile && fclose(basefile) == EOF && err == NULL)
 		err = got_error_from_errno("fclose");
 	if (accumfile && fclose(accumfile) == EOF && err == NULL)
@@ -2164,6 +2179,7 @@ main(int argc, char *argv[])
 			got_privsep_send_error(&ibuf, err);
 		}
 	}
+	imsgbuf_clear(&ibuf);
 	if (close(GOT_IMSG_FD_CHILD) == -1 && err == NULL)
 		err = got_error_from_errno("close");
 	return err ? 1 : 0;
