@@ -55,10 +55,10 @@ test_histedit_no_op() {
 	local new_commit2=`git_show_head $testroot/repo`
 	local new_author_time2=`git_show_author_time $testroot/repo`
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
-	local short_new_commit1=`trim_obj_id 28 $new_commit1`
-	local short_new_commit2=`trim_obj_id 28 $new_commit2`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
+	local short_new_commit1=`trim_obj_id 12 $new_commit1`
+	local short_new_commit2=`trim_obj_id 12 $new_commit2`
 
 	echo "G  alpha" > $testroot/stdout.expected
 	echo "D  beta" >> $testroot/stdout.expected
@@ -267,10 +267,10 @@ test_histedit_swap() {
 	local new_commit2=`git_show_parent_commit $testroot/repo`
 	local new_commit1=`git_show_head $testroot/repo`
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
-	local short_new_commit1=`trim_obj_id 28 $new_commit1`
-	local short_new_commit2=`trim_obj_id 28 $new_commit2`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
+	local short_new_commit1=`trim_obj_id 12 $new_commit1`
+	local short_new_commit2=`trim_obj_id 12 $new_commit2`
 
 	echo "G  epsilon/zeta" > $testroot/stdout.expected
 	echo -n "$short_old_commit2 -> $short_new_commit2: " \
@@ -388,9 +388,9 @@ test_histedit_drop() {
 
 	local new_commit2=`git_show_head $testroot/repo`
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
-	local short_new_commit2=`trim_obj_id 28 $new_commit2`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
+	local short_new_commit2=`trim_obj_id 12 $new_commit2`
 
 	echo "$short_old_commit1 ->  drop commit: committing changes" \
 		> $testroot/stdout.expected
@@ -511,11 +511,11 @@ EOF
 	local new_commit1=`git_show_parent_commit $testroot/repo`
 	local new_commit2=`git_show_head $testroot/repo`
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
-	local short_old_commit3=`trim_obj_id 28 $old_commit3`
-	local short_new_commit1=`trim_obj_id 28 $new_commit1`
-	local short_new_commit2=`trim_obj_id 28 $new_commit2`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
+	local short_old_commit3=`trim_obj_id 12 $old_commit3`
+	local short_new_commit1=`trim_obj_id 12 $new_commit1`
+	local short_new_commit2=`trim_obj_id 12 $new_commit2`
 
 	echo "G  alpha" > $testroot/stdout.expected
 	echo "D  beta" >> $testroot/stdout.expected
@@ -617,8 +617,8 @@ test_histedit_edit() {
 	(cd $testroot/wt && got histedit -F $testroot/histedit-script \
 		> $testroot/stdout)
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
 
 	echo "G  alpha" > $testroot/stdout.expected
 	echo "D  beta" >> $testroot/stdout.expected
@@ -675,8 +675,8 @@ EOF
 	local new_commit1=`git_show_parent_commit $testroot/repo`
 	local new_commit2=`git_show_head $testroot/repo`
 
-	local short_new_commit1=`trim_obj_id 28 $new_commit1`
-	local short_new_commit2=`trim_obj_id 28 $new_commit2`
+	local short_new_commit1=`trim_obj_id 12 $new_commit1`
+	local short_new_commit2=`trim_obj_id 12 $new_commit2`
 
 	echo "$short_old_commit1 -> $short_new_commit1: committing changes" \
 		> $testroot/stdout.expected
@@ -926,8 +926,8 @@ test_histedit_abort() {
 	(cd $testroot/wt && got histedit -F $testroot/histedit-script \
 		> $testroot/stdout)
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
 
 	echo "G  alpha" > $testroot/stdout.expected
 	echo "D  beta" >> $testroot/stdout.expected
@@ -1058,8 +1058,8 @@ test_histedit_path_prefix_drop() {
 	(cd $testroot/wt && got histedit -F $testroot/histedit-script \
 		> $testroot/stdout)
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
 
 	echo "$short_old_commit1 ->  drop commit: changing zeta" \
 		> $testroot/stdout.expected
@@ -1161,7 +1161,7 @@ test_histedit_path_prefix_edit() {
 	(cd $testroot/wt && got histedit -F $testroot/histedit-script \
 		> $testroot/stdout)
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
 
 	echo "G  zeta" > $testroot/stdout.expected
 	echo "Stopping histedit for amending commit $old_commit1" \
@@ -1212,7 +1212,7 @@ EOF
 		got histedit -c > $testroot/stdout)
 
 	local new_commit1=`git_show_head $testroot/repo`
-	local short_new_commit1=`trim_obj_id 28 $new_commit1`
+	local short_new_commit1=`trim_obj_id 12 $new_commit1`
 
 	echo -n "$short_old_commit1 -> $short_new_commit1: " \
 		> $testroot/stdout.expected
@@ -1359,9 +1359,9 @@ EOF
 
 	local new_commit=`git_show_head $testroot/repo`
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
-	local short_new_commit=`trim_obj_id 28 $new_commit`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
+	local short_new_commit=`trim_obj_id 12 $new_commit`
 
 	echo "G  epsilon/zeta" >> $testroot/stdout.expected
 	echo -n "$short_old_commit2 ->  fold commit: committing to zeta " \
@@ -1395,12 +1395,12 @@ test_histedit_split_commit() {
 	git -C $testroot/repo add epsilon/new
 	git_commit $testroot/repo -m "committing changes 1"
 	local old_commit1=`git_show_head $testroot/repo`
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
 
 	echo "modified zeta on master" > $testroot/repo/epsilon/zeta
 	git_commit $testroot/repo -m "committing changes 2"
 	local old_commit2=`git_show_head $testroot/repo`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
 
 	got checkout -c $orig_commit $testroot/repo $testroot/wt > /dev/null
 	ret=$?
@@ -1471,7 +1471,7 @@ test_histedit_split_commit() {
 		return 1
 	fi
 	local new_commit2=`git_show_head $testroot/repo`
-	local short_new_commit2=`trim_obj_id 28 $new_commit2`
+	local short_new_commit2=`trim_obj_id 12 $new_commit2`
 
 	echo "$short_old_commit1 -> no-op change: committing changes 1" \
 		> $testroot/stdout.expected
@@ -1587,10 +1587,10 @@ EOF
 
 	local new_commit1=`git_show_head $testroot/repo`
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
-	local short_old_commit3=`trim_obj_id 28 $old_commit3`
-	local short_new_commit1=`trim_obj_id 28 $new_commit1`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
+	local short_old_commit3=`trim_obj_id 12 $old_commit3`
+	local short_new_commit1=`trim_obj_id 12 $new_commit1`
 
 	echo "A  epsilon/psi" >> $testroot/stdout.expected
 	echo "$short_old_commit1 ->  fold commit: committing changes" \
@@ -1690,9 +1690,9 @@ EOF
 
 	local new_commit1=`git_show_head $testroot/repo`
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
-	local short_new_commit1=`trim_obj_id 28 $new_commit1`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
+	local short_new_commit1=`trim_obj_id 12 $new_commit1`
 
 	echo "G  alpha" >> $testroot/stdout.expected
 	echo "$short_old_commit1 ->  fold commit: modified alpha" \
@@ -1779,9 +1779,9 @@ EOF
 
 	local new_commit1=`git_show_head $testroot/repo`
 	
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
-	local short_new_commit1=`trim_obj_id 28 $new_commit1`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
+	local short_new_commit1=`trim_obj_id 12 $new_commit1`
 
 	echo "D  alpha" > $testroot/stdout.expected
 	echo "$short_old_commit1 ->  fold commit: removing alpha" \
@@ -1859,11 +1859,11 @@ EOF
 
 	local new_commit1=`git_show_head $testroot/repo`
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
-	local short_old_commit3=`trim_obj_id 28 $old_commit3`
-	local short_new_commit1=`trim_obj_id 28 $new_commit1`
-	local short_new_commit2=`trim_obj_id 28 $new_commit2`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
+	local short_old_commit3=`trim_obj_id 12 $old_commit3`
+	local short_new_commit1=`trim_obj_id 12 $new_commit1`
+	local short_new_commit2=`trim_obj_id 12 $new_commit2`
 
 	echo "G  alpha" > $testroot/stdout.expected
 	echo "D  beta" >> $testroot/stdout.expected
@@ -1978,12 +1978,12 @@ EOF
 
 	local new_commit1=`git_show_head $testroot/repo`
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local very_short_old_commit1=`trim_obj_id 29 $old_commit1`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
-	local short_old_commit3=`trim_obj_id 28 $old_commit3`
-	local short_new_commit1=`trim_obj_id 28 $new_commit1`
-	local short_new_commit2=`trim_obj_id 28 $new_commit2`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local very_short_old_commit1=`trim_obj_id 11 $old_commit1`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
+	local short_old_commit3=`trim_obj_id 12 $old_commit3`
+	local short_new_commit1=`trim_obj_id 12 $new_commit1`
+	local short_new_commit2=`trim_obj_id 12 $new_commit2`
 
 	echo "G  alpha" > $testroot/stdout.expected
 	echo "D  beta" >> $testroot/stdout.expected
@@ -2083,8 +2083,8 @@ test_histedit_edit_only() {
 
 	(cd $testroot/wt && got histedit -e > $testroot/stdout)
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
 
 	echo "G  alpha" > $testroot/stdout.expected
 	echo "D  beta" >> $testroot/stdout.expected
@@ -2115,7 +2115,7 @@ EOF
 
 	local new_commit1=$(cd $testroot/wt && got info | \
 		grep '^work tree base commit: ' | cut -d: -f2 | tr -d ' ')
-	local short_new_commit1=`trim_obj_id 28 $new_commit1`
+	local short_new_commit1=`trim_obj_id 12 $new_commit1`
 
 	echo -n "$short_old_commit1 -> $short_new_commit1: " \
 		> $testroot/stdout.expected
@@ -2146,7 +2146,7 @@ EOF
 		VISUAL="$testroot/editor.sh" got histedit -c > $testroot/stdout)
 
 	local new_commit2=`git_show_head $testroot/repo`
-	local short_new_commit2=`trim_obj_id 28 $new_commit2`
+	local short_new_commit2=`trim_obj_id 12 $new_commit2`
 
 	echo -n "$short_old_commit2 -> $short_new_commit2: " \
 		> $testroot/stdout.expected
@@ -2229,7 +2229,7 @@ EOF
 	(cd $testroot/wt/ && got commit -m 'modified alpha on master' \
 		alpha > /dev/null)
 	ret=$?
-	if [ "$?" != 0 ]; then
+	if [ $ret -ne 0 ]; then
 		echo "got commit failed unexpectedly" >&2
 		test_done "$testroot" "$ret"
 		return 1
@@ -2240,7 +2240,7 @@ EOF
 
 	(cd $testroot/wt/ && got update -c $orig_commit > /dev/null)
 	ret=$?
-	if [ "$?" != 0 ]; then
+	if [ $ret -ne 0 ]; then
 		echo "got update failed unexpectedly" >&2
 		test_done "$testroot" "$ret"
 		return 1
@@ -2249,7 +2249,7 @@ EOF
 	(cd $testroot/wt && got histedit -F "$testroot/histedit-script" \
 		> /dev/null)
 	ret=$?
-	if [ "$?" != 0 ]; then
+	if [ $ret -ne 0 ]; then
 		echo "got histedit failed unexpectedly" >&2
 		test_done "$testroot" "$ret"
 		return 1
@@ -2279,7 +2279,7 @@ test_histedit_resets_committer() {
 	(cd $testroot/wt/ && got commit -m 'modified alpha on master' \
 		alpha > /dev/null)
 	ret=$?
-	if [ "$?" != 0 ]; then
+	if [ $ret -ne 0 ]; then
 		echo "got commit failed unexpectedly" >&2
 		test_done "$testroot" "$ret"
 		return 1
@@ -2290,7 +2290,7 @@ test_histedit_resets_committer() {
 
 	(cd $testroot/wt/ && got update -c $orig_commit > /dev/null)
 	ret=$?
-	if [ "$?" != 0 ]; then
+	if [ $ret -ne 0 ]; then
 		echo "got update failed unexpectedly" >&2
 		test_done "$testroot" "$ret"
 		return 1
@@ -2299,7 +2299,7 @@ test_histedit_resets_committer() {
 	(cd $testroot/wt && env GOT_AUTHOR="$committer" \
 		got histedit -F "$testroot/histedit-script" > /dev/null)
 	ret=$?
-	if [ "$?" != 0 ]; then
+	if [ $ret -ne 0 ]; then
 		echo "got histedit failed unexpectedly" >&2
 		test_done "$testroot" "$ret"
 		return 1
@@ -2437,8 +2437,8 @@ EOF
 	local new_commit1=`git_show_head $testroot/repo`
 	local new_author_time1=`git_show_author_time $testroot/repo`
 
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
-	local short_new_commit1=`trim_obj_id 28 $new_commit1`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
+	local short_new_commit1=`trim_obj_id 12 $new_commit1`
 
 	echo "G  alpha" > $testroot/stdout.expected
 	echo "$short_old_commit1 -> $short_new_commit1: set executable bit on alpha" \
@@ -2530,9 +2530,9 @@ test_histedit_drop_only() {
 	(cd $testroot/wt && got histedit -d > $testroot/stdout)
 	local new_commit1=`git_show_head $testroot/repo`
 
-	local short_commit1=`trim_obj_id 28 $drop_commit1`
-	local short_commit2=`trim_obj_id 28 $drop_commit2`
-	local short_commit3=`trim_obj_id 28 $drop_commit3`
+	local short_commit1=`trim_obj_id 12 $drop_commit1`
+	local short_commit2=`trim_obj_id 12 $drop_commit2`
+	local short_commit3=`trim_obj_id 12 $drop_commit3`
 
 	echo "$short_commit1 $drop $dropmsg 1" > $testroot/stdout.expected
 	echo "$short_commit2 $drop $dropmsg 2" >> $testroot/stdout.expected
@@ -2618,17 +2618,17 @@ test_histedit_conflict_revert() {
 	echo "first change of alpha" > $testroot/repo/alpha
 	git_commit $testroot/repo -m "committing changes"
 	local old_commit1=`git_show_head $testroot/repo`
-	local short_old_commit1=`trim_obj_id 28 $old_commit1`
+	local short_old_commit1=`trim_obj_id 12 $old_commit1`
 
 	echo "second change of alpha" > $testroot/repo/alpha
 	git_commit $testroot/repo -m "committing changes"
 	local old_commit2=`git_show_head $testroot/repo`
-	local short_old_commit2=`trim_obj_id 28 $old_commit2`
+	local short_old_commit2=`trim_obj_id 12 $old_commit2`
 
 	echo "third change of alpha" > $testroot/repo/alpha
 	git_commit $testroot/repo -m "committing changes"
 	local old_commit3=`git_show_head $testroot/repo`
-	local short_old_commit3=`trim_obj_id 28 $old_commit3`
+	local short_old_commit3=`trim_obj_id 12 $old_commit3`
 
 	got checkout -c $orig_commit $testroot/repo $testroot/wt > /dev/null
 	ret=$?
@@ -2672,7 +2672,7 @@ EOF
 
 	local new_commit1=$(cd $testroot/wt && got info | \
 		grep 'work tree base commit:' | cut -d ' ' -f5)
-	local short_new_commit1=`trim_obj_id 28 $new_commit1`
+	local short_new_commit1=`trim_obj_id 12 $new_commit1`
 
 	echo "$short_old_commit1 -> $short_new_commit1: committing changes" \
 		> $testroot/stdout.expected
@@ -2758,6 +2758,150 @@ EOF
 	test_done "$testroot" "$ret"
 }
 
+test_histedit_no_eof_newline() {
+	local testroot=$(test_init histedit_no_eof_newline)
+	local id_root=$(git_show_head "$testroot/repo")
+
+	got checkout "$testroot/repo" "$testroot/wt" > /dev/null
+
+	echo -n "fubar" >> $testroot/wt/alpha
+
+	(cd "$testroot/wt" && got commit -m 'no eof newline' > /dev/null)
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		echo "got commit failed unexpectedly" >&2
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	cp $testroot/wt/alpha $testroot/content.expected
+
+	(cd $testroot/wt && got update -c :base:- > /dev/null)
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		echo "update to $id_root failed" >&2
+		test_done "$testroot" 1
+		return 1
+	fi
+
+	cat > $testroot/editor.sh <<EOF
+#!/bin/sh
+ed -s "\$1" <<-EOF
+	,s/ eof / terminating /
+	w
+	EOF
+EOF
+	chmod +x $testroot/editor.sh
+
+	(cd $testroot/wt && env EDITOR="$testroot/editor.sh" \
+	    VISUAL="$testroot/editor.sh" got histedit -m > /dev/null)
+
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		echo "histedit -m failed" >&2
+		test_done "$testroot" $ret
+		return 1
+	fi
+
+	cp $testroot/wt/alpha $testroot/content
+
+	cmp -s $testroot/content.expected $testroot/content
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		diff -u $testroot/content.expected $testroot/content
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	cat > $testroot/wt/beta <<-EOF
+	beta
+	2
+	3
+	4
+
+	6
+	7
+	8
+	EOF
+
+	(cd "$testroot/wt" && got commit -m 'padding beta' > /dev/null)
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		echo "got commit failed unexpectedly" >&2
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	local id=$(git_show_head "$testroot/repo")
+
+	cat > $testroot/wt/beta <<-EOF
+	betazoid
+	3
+	4
+
+	7
+	8
+	a
+	b
+	c
+	d
+	e
+	f
+	0
+	1
+	2
+	3
+	4
+	5
+	6
+	7
+	8
+	9
+	EOF
+
+	echo -n "fubar" >> $testroot/wt/beta
+
+	(cd "$testroot/wt" && got commit -m 'beta no eof newline' > /dev/null)
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		echo "got commit failed unexpectedly" >&2
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	cp $testroot/wt/beta $testroot/content.expected
+
+	(cd $testroot/wt && got update -c :base:- > /dev/null)
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		echo "update to $id failed" >&2
+		test_done "$testroot" 1
+		return 1
+	fi
+
+	(cd $testroot/wt && env EDITOR="$testroot/editor.sh" \
+	    VISUAL="$testroot/editor.sh" got histedit -m > /dev/null)
+
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		echo "histedit -m failed" >&2
+		test_done "$testroot" $ret
+		return 1
+	fi
+
+	cp $testroot/wt/beta $testroot/content
+
+	cmp -s $testroot/content.expected $testroot/content
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		diff -u $testroot/content.expected $testroot/content
+		test_done "$testroot" "$ret"
+		return 1
+	fi
+
+	test_done "$testroot" 0
+}
+
 test_parseargs "$@"
 run_test test_histedit_no_op
 run_test test_histedit_swap
@@ -2786,3 +2930,4 @@ run_test test_histedit_umask
 run_test test_histedit_mesg_filemode_change
 run_test test_histedit_drop_only
 run_test test_histedit_conflict_revert
+run_test test_histedit_no_eof_newline
